body {
  font-family: sans-serif;
  background: white;
  color: black;
  margin: 0px;
  padding: 15px;
}

a:link {
  font-weight: bold;
  text-decoration: none;
  color: gray;
}

a:visited {
  font-weight: bold;
  text-decoration: none;
  color: gray;
}

a:hover {
  color: #cccccc;
  text-decoration: underline;
}

a:active {
  color: #cccccc;
  text-decoration: underline;
}

img { 
  border-style: none;
}

h1 {
  font-family: sans-serif;
}

h2 {
  font-family: sans-serif;
}

h3 {
  font-family: sans-serif;
}

/* container */

#container {
  position: absolute;
  border-width: thin;
  border-style: solid;
  width: 95%;
}

/* intro */

#intro {
  padding: 5px;
  margin: 0px;
  background: #cccccc;
  border-width: medium;
  border-style: solid;
}

#intro h1 {
  margin: 5px;
  padding: 5px;
}

/* links */

#links { 
  font-size: x-small;
  vertical-align: bottom;
}

#links a {
  border-width: thin;
  border-style: dotted;
  border-color: white;
  /* margin: 0px 10px 0px 0px; */
  margin: 1px;
  padding: 3px;
  line-height: 230%
}

#links a:hover {
  color: black;
  text-decoration: underline;
}

#links h3 {
  outline-width: thin;
  border-style: solid;
  padding: 2px;
  margin: 3px 0px 3px 0px;
}

/* menu */

#menu h3 {
  text-align: center;
}

/* text */

#text {
  margin: 0px;
  padding: 5px 5px 0px 5px;
  float: left;
}

#text h3 {
  border-width: thin;
  border-style: solid;
  padding: 2px;
  margin: 3px 0px 3px 0px;
}

#text li {
  margin: 0px 0px 10px 0px;
}

#text ul {
  margin: 5px;
  padding: 0px 0px 0px 20px;
}

#leftcolumn {
  float: left;
  width: 300px;
  margin: 0px 10px 0px 0px;
  padding: 0px;
}

#rightcolumn {
  float: right;
  width: 210px;
  margin: 0px;
  padding: 0px;
}

/* vspacer */

.vspacer {
  height: 10px;
}

.silver {
  border-width: thin;
  border-color: black;
  border-style: solid;
  background: #cccccc;
}

a.code {
  text-decoration: none;
  font-weight: normal;
  color: #4444ee
}

a.codeRef {
  font-weight: normal;
  color: #4444ee
}

div.fragment {
  width: 98%;
  border: 1px solid #CCCCCC;
  background-color: #f5f5f5;
  padding-left: 4px;
  margin: 4px;
}

div.ah {
  background-color: black;
  font-weight: bold; color: #ffffff;
  margin-bottom: 3px;
  margin-top: 3px
}

#text td {
  width: auto;
}

div.memdoc {
  margin-top: 0px;
  margin-bottom: 20px;
  padding: 10px 10px 10px 40px;
}

div.memproto {
  border: thin solid black;
  background-color: #f2f2ff;
  width: 100%;
  margin-top: 20px;
  padding-top: 10px;
  padding-bottom: 10px;
}

td.paramtype {
  color: #602020;
}

table.memname {
  font-weight: bold;
}

div.groupHeader {
  margin-left: 16px;
  margin-top: 12px;
  margin-bottom: 6px;
  font-weight: bold
}

div.groupText {
  margin-left: 16px;
  font-style: italic;
  font-size: smaller
}

body {
  background: white;
  color: black;
  margin-right: 20px;
  margin-left: 20px;
}

td.indexkey { 
  background-color: #eeeeff; 
  font-weight: bold; 
  padding-right  : 10px; 
  padding-top    : 2px; 
  padding-left   : 10px; 
  padding-bottom : 2px; 
  margin-left    : 0px; 
  margin-right   : 0px; 
  margin-top     : 2px; 
  margin-bottom  : 2px  
}

td.indexvalue { 
  background-color: #eeeeff; 
  font-style: italic; 
  padding-right  : 10px; 
  padding-top    : 2px; 
  padding-left   : 10px; 
  padding-bottom : 2px; 
  margin-left    : 0px; 
  margin-right   : 0px; 
  margin-top     : 2px; 
  margin-bottom  : 2px  
}

tr.memlist {
  background-color: #f0f0f0; 
}

p.formulaDsp {
  text-align: center;
}

img.formulaDsp {

}

img.formulaInl {
  vertical-align: middle;
}

span.keyword {
  color: #008000
}

span.keywordtype {
  color: #604020
}

span.keywordflow {
  color: #e08000
}

span.comment {
  color: #800000
}

span.preprocessor {
  color: #806020
}

span.stringliteral {
  color: #002080
}

span.charliteral {
  color: #008080
}

.mdTable {
  border: 1px solid #868686;
  background-color: #f2f2ff;
}

.mdRow {
  padding: 8px 20px;
}

.mdescLeft {
  font-size: smaller;
  font-family: Arial, Helvetica, sans-serif;
  background-color: #FAFAFA;
  padding-left: 8px;
  border-top: 1px none #E0E0E0;
  border-right: 1px none #E0E0E0;
  border-bottom: 1px none #E0E0E0;
  border-left: 1px none #E0E0E0;
  margin: 0px;
}

.mdescRight {
  font-size: smaller;
  font-family: Arial, Helvetica, sans-serif;
  font-style: italic;
  background-color: #FAFAFA;
  padding-left: 4px;
  border-top: 1px none #E0E0E0;
  border-right: 1px none #E0E0E0;
  border-bottom: 1px none #E0E0E0;
  border-left: 1px none #E0E0E0;
  margin: 0px;
  padding-bottom: 0px;
  padding-right: 8px;
}

.memItemLeft {
  padding: 1px 0px 0px 8px;
  margin: 4px;
  border-top-width: 1px;
  border-right-width: 1px;
  border-bottom-width: 1px;
  border-left-width: 1px;
  border-top-style: solid;
  border-top-color: #E0E0E0;
  border-right-color: #E0E0E0;
  border-bottom-color: #E0E0E0;
  border-left-color: #E0E0E0;
  border-right-style: none;
  border-bottom-style: none;
  border-left-style: none;
  background-color: #FAFAFA;
  font-family: Geneva, Arial, Helvetica, sans-serif;
  font-size: 12px;
}

.memItemRight {
  padding: 1px 0px 0px 8px;
  margin: 4px;
  border-top-width: 1px;
  border-right-width: 1px;
  border-bottom-width: 1px;
  border-left-width: 1px;
  border-top-style: solid;
  border-top-color: #E0E0E0;
  border-right-color: #E0E0E0;
  border-bottom-color: #E0E0E0;
  border-left-color: #E0E0E0;
  border-right-style: none;
  border-bottom-style: none;
  border-left-style: none;
  background-color: #FAFAFA;
  font-family: Geneva, Arial, Helvetica, sans-serif;
  font-size: 13px;
}

.search {
  color: #0000ee;
  font-weight: bold;
}

form.search {
  margin-bottom: 0px;
  margin-top: 0px;
}

input.search {
  font-size: 75%;
  color: #000080;
  font-weight: normal;
  background-color: #eeeeff;
}

td.tiny {
  font-size: 75%;
}
